-- Project: Fractal for Spartan-3
-- Author : Matthias Alles
-- Mail   : alles(at) rhrk.uni-kk.de
--
-- Simple VGA timing generation.
-- Generates a 640*480 screen at 60Hz
-- The pixelclock was 24,576MHz.
-- Some day I will do this generic I think.
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity vga_timing is port (
		clk   : in std_logic;
		rst   : in std_logic;
		hsync : out std_logic;
		vsync : out std_logic;
		blank : out std_logic;
		x     : out std_logic_vector(9 downto 0);
		y     : out std_logic_vector(9 downto 0));
	end vga_timing;

architecture rtl of vga_timing is

	signal x_counter: unsigned(9 downto 0);
	signal y_counter: unsigned(9 downto 0);

begin

	x <= std_logic_vector(x_counter);
	y <= std_logic_vector(y_counter);

	generate_timing: process(clk, rst) is
	begin
		if rst = '1' then
			x_counter <= (others => '0');
			y_counter <= (others => '0');
			hsync     <= '1';
			vsync     <= '1';
			blank     <= '0';
		elsif clk'event and clk = '1' then

			-- handle counters
			x_counter <= x_counter + 1;
		
			if x_counter = 779 then
				x_counter <= (others => '0');
				y_counter <= y_counter + 1;
				if y_counter = 524 then
					y_counter <= (others => '0');
				end if;
			end if;

			-- generate timing signals
			if x_counter >= 640 then  --512
				blank <= '1';
				if x_counter >= 663 and x_counter < 757 then  -- 524, 598
					hsync <= '0';
				else
					hsync <= '1';
				end if;
			else
				blank <= '0';
				hsync <= '1';
			end if;


			if y_counter >= 480 then	 -- vertical sync in action
				blank <= '1';
				if y_counter = 490 or y_counter = 491 then
					vsync <= '0';
				else
					vsync <= '1';
				end if;
			end if;

		end if;
	end process generate_timing;

end rtl;
